import * as X_STATE from 'xstate';

// const X = X_STATE.createMachine({
//     id: 'X',
//     initial: 'A',
//     type: "parallel",
//     states: {
//         A: { // Sending events like this fails: 'entry: () => X_STATE.raise('e2')'. Solution:
//             entry: {type: 'xstate.raise', event: {type: 'e2'}}
//         },
//         B: {
//             on: {
//                 e2: {target: 'B'}
//             },
//             entry: () => console.info("\tEntering 'B'... again?")
//         }
//     }
// });
// (X_STATE.interpret(X).onTransition((state) => {
//     console.log(state.value);
// })).start();

interface Local_variables {
    counter: number;
}

const my_device: any = X_STATE.createMachine<Local_variables>({
        context: {
            counter: 0
        },
        id: 'My_device',
        initial: 'Idle',
        predictableActionArguments: true, // Default option from v. 5.x...
        states: {
            Idle: {
                id: 'Idle',
                on: {
                    go: 'Busy'
                }
            },
            Busy: {
                on: {
                    request_c: {
                        target: ['Busy.S1.S12', 'Busy.S2.S22'],
                    },
                    request_d: {
                        target: 'Busy.S2.S21',
                    },
                    request_e: {
                        target: 'Idle',
                    }
                },
                type: 'parallel',
                states: {
                    S1: {
                        initial: 'S11',
                        on: {
                            request_b: 'S1.S12'
                        },
                        states: {
                            S11: {
                                entry: ['w'],
                                exit: ['x'],
                            },
                            S12: {
                                entry: ['y'],
                                exit: ['z'],
                                on: {
                                    request_g: 'S11'
                                }
                            }
                        }
                    },
                    S2: {
                        initial: 'S22',
                        states: {
                            S21: {
                                // https://xstate.js.org/docs/guides/transitions.html#faq-s
                                on: { // Use of state id.:
                                    request_f: '#Idle'
                                }
                            },
                            S22: {
                                entry: {type: 'xstate.raise', event: {type: "request_h"}}
                                // To be avoided: 'entry: () => instance.send("request_h")'
                            }
                        }
                    },
                    S3: {
                        type: 'parallel',
                        states: {
                            S31: {
                                entry: () => console.info("Entering 'S31'..."),
                                on: {
                                    request_h: {
                                        target: 'S31',
                                        actions: 'a' // https://xstate.js.org/docs/guides/actions.html#api
                                    }
                                },
                            },
                            S32: {}
                        }
                    }
                }
            }
        }
    },
    {
        actions: {
            a: () => console.info('\ta'),
            w: () => console.info('\tw'),
            x: () => console.info('\tx'),
            y: () => console.info('\ty'),
            z: () => console.info('\tz')
        },
    });

const instance = X_STATE.interpret(my_device).onTransition((state, event) => {
    console.log(state.value);
    console.log(event.type);
});
instance.start();
instance.send("go");
// instance.send("request_d");
// instance.send("request_c");
// instance.send("request_d");
// instance.send("request_f");
// instance.stop();

